import React from 'react';
import * as PropTypes from 'prop-types';

// Header component to be used as default for all the columns.
export default class SortableHeaderComponent extends React.Component {
    constructor(props) {
        super(props);

        // The state of this component contains the current sort state of this column
        // The possible values are: 'asc', 'desc' and ''
        this.state = {
            sorted: '',
            isFilterShowing: false,
        };
    }

    render() {
        let pinned = false;
        const {
            columnList,
            displayName,
            enableSorting,
        } = this.props;

        columnList.forEach((col) => {
            if ((col.headerName === displayName) && col.pinned) {
                pinned = true;
            }
            else if (col.headerName === displayName && !col.pinned) {
                pinned = false;
            }
        });
        const sortElements = [];
        if (enableSorting) {
            const { sorted } = this.state;
            const downArrowClass = 'customSortDownLabel ' + (sorted === 'desc' ? ' active' : '');
            const upArrowClass = 'customSortUpLabel ' + (sorted === 'asc' ? ' active' : '');
            const removeArrowClass = 'customSortRemoveLabel ' + (sorted === '' ? ' active' : '');

            sortElements.push(
                <div
                    role="presentation"
                    key={ `up${displayName}` }
                    className={ downArrowClass }
                    onClick={ () => this.handleSortRequested('desc') }>
                    <i className="fa fa-long-arrow-alt-down" />
                </div>);
            sortElements.push(
                <div
                    role="presentation"
                    key={ `down${displayName}` }
                    className={ upArrowClass }
                    onClick={ () => this.handleSortRequested('asc') }>
                    <i className="fa fa-long-arrow-alt-up" />
                </div>);
            sortElements.push(
                <div
                    role="presentation"
                    key={ `minus${displayName}` }
                    className={ removeArrowClass }
                    onClick={ () => this.handleSortRequested('') }>
                    <i className="fa fa-times" />
                </div>);
        }

        const pin = (
            <div
                role="presentation"
                onClick={ () => this.handlePinClick(displayName) }
                key={ `pin${displayName}` }
                className="pinContainer">
                <i
                    style={ { paddingLeft: '4px' } }
                    className="fas fa-thumbtack" />
            </div>
        );

        const pinInactive = (
            <div
                role="presentation"
                onClick={ () => this.handlePinClick(displayName) }
                key={ `pinInactive${displayName}` }
                className="pinInactiveContainer">
                <i
                    style={ { paddingLeft: '4px' } }
                    className="fa fa-thumbtack" />
            </div>
        );

        return (
            <div className="customHeaderContents">
                <div
                    role="presentation"
                    onClick={ this.handleFilterToggle }
                    className="customHeaderLabel">
                    { displayName }
                </div>
                { sortElements }
                { pinned ? pin : pinInactive }
            </div>
        );
    }

    /**
     * Toggles between show/hide state for filter dialog
     */
    handleFilterToggle = () => {
        const { handleFilterToggle } = this.props;
        const { isFilterShowing } = this.state;
        this.setState({ isFilterShowing: !isFilterShowing }, handleFilterToggle(!isFilterShowing));
    }

    /**
     * Passes column info for column on which pin icon was clicked.
     * @method handlePinClick
     * @param {string} data column identifier
     */
    handlePinClick = (data) => {
        const { handlePinClick } = this.props;
        if (handlePinClick) {
            handlePinClick(data);
        }
    }

    /**
     * Handles sorting on columns
     * @method onSortRequested
     * @param {string} order Sorting order, 'asc', 'desc' or ''
     */
    handleSortRequested = (order) => {
        const { onSortRequested } = this.props;
        if (onSortRequested) {
            onSortRequested(order)
                .then((sortOrder) => {
                    this.setState({ sorted: sortOrder });
                });
        }
    };
}

SortableHeaderComponent.propTypes = {
    /** Method to be called on click of pin icon */
    handlePinClick: PropTypes.func,

    /** Handles sorting of rows */
    onSortRequested: PropTypes.func,

    /** Handles show/hide toggle for filterDialog */
    handleFilterToggle: PropTypes.func,

    /** Name of column header */
    displayName: PropTypes.string,

    /** Metadata for columns */
    columnList: PropTypes.arrayOf(PropTypes.object),

    enableSorting: PropTypes.bool,
};