import React from 'react';
import MUIDialog from '@material-ui/core/Dialog';
import * as PropTypes from 'prop-types';

import { makeStyles } from '@material-ui/core/styles';
import getStyles from './Modal.styles';

/**
 * A modal that displays content in the middle of the entire viewport and over all other content.
 * @see https://material-ui.com/components/dialogs/
 * @see https://material-ui.com/api/dialog/
 */
const Modal = (props) => {
    const {
        children,
        contextStyles,
        isClosable,
        isModalOpen,
        isTransparentOverlay,
        onClose,
        testID,
    } = props;

    // Override core styles with context styles, separating MUI styles
    const muiStyles = getStyles(contextStyles, isTransparentOverlay);

    // Create dynamic class names and injected DOM styles for MUI component
    // * makeStyles returns a function
    const styles = makeStyles(muiStyles)();

    // Generate dialog props
    const dialogProps = {
        className: 'c-modal',
        classes: {
            container: styles.overlay, // MUI styles
            paper: styles.modal, // MUI styles
        },
        disableBackdropClick: isClosable,
        onClose,
        open: isModalOpen,
        scroll: 'paper',
        BackdropProps: { invisible: true },
    };
    if (testID) { dialogProps['data-test-id'] = `${testID}__modal`; }

    return (
        <MUIDialog
            { ...dialogProps }
            aria-labelledby="alert-dialog-title"
            aria-describedby="alert-dialog-description">
            { children }
        </MUIDialog>
    );
};

Modal.propTypes = {
    /** Modal contents */
    children: PropTypes.node,

    /** Styles for this component */
    contextStyles: PropTypes.shape({
        /** Styles for the modal container */
        modal: PropTypes.object,

        /** Styles for the overlay */
        overlay: PropTypes.object,
    }),

    /** Disable backdrop click modal closing */
    isClosable: PropTypes.bool,

    /** Defines open or closed state of modal */
    isModalOpen: PropTypes.bool,

    /** Changes the overlay darkened fill to transparent */
    isTransparentOverlay: PropTypes.bool,

    /** Called when modal is closable and user closes the modal. */
    onClose: PropTypes.func,

    /** Id used for testing */
    testID: PropTypes.string,
};

Modal.defaultProps = {
    contextStyles: {},
    isClosable: true,
};

export default Modal;
